const path = require('path');
const webpack = require('webpack');
const webpackDevMiddleware = require('webpack-dev-middleware');
const webpackHotMiddleware = require('webpack-hot-middleware');
const proxy = require('express-http-proxy');

function createWebpackMiddleware(compiler, publicPath) {
  return webpackDevMiddleware(compiler, {
    logLevel: 'warn',
    publicPath,
    silent: true,
    stats: 'errors-only',
  });
}

module.exports = function addDevMiddlewares(app, webpackConfig) {
  const compiler = webpack(webpackConfig);
  const middleware = createWebpackMiddleware(
    compiler,
    webpackConfig.output.publicPath,
  );

  app.use(middleware);
  app.use(webpackHotMiddleware(compiler));

  // Since webpackDevMiddleware uses memory-fs internally to store build
  // artifacts, we use it instead
  const fs = middleware.fileSystem;
  let proxyHost = 'localhost:3002';
  app.get('/setproxy', (req, res) => {
    if (req.query && req.query.host) {
      proxyHost = req.query.host;
      res.send(200);
    } else {
      res.send(400);
    }
  });
  app.use(
    '/proxy/*',
    proxy(() => proxyHost, {
      proxyReqPathResolver: req => {
        const parts = req.originalUrl.split('?');
        const queryString = parts[1];
        const updatedPath = parts[0].replace(/\/proxy\//, '/');
        return updatedPath + (queryString ? `?${queryString}` : '');
      },
      proxyReqOptDecorator: proxyReqOpts => {
        // you can update headers
        proxyReqOpts.headers.origin = proxyHost;
        // proxyReqOpts.headers.Accept = '*/*';
        // proxyReqOpts.headers['cookie'] =
        //   'UM_distinctid=17543f1ba9ea0a-05f0523c41099d-c781f38-1fa400-17543f1ba9f59; CNZZDATA1278602214=833011536-1603161215-%7C1603161215';
        // proxyReqOpts.headers['Accept-Encoding'] = 'gzip, deflate, br';
        // proxyReqOpts.headers['Accept-Language'] =
        //   'zh-CN,zh;q=0.9,zh-TW;q=0.8,en-US;q=0.7,en;q=0.6';
        // console.log({ proxyReqOpts });
        // you can change the method
        return proxyReqOpts;
      },
    }),
  );

  app.get('*', (req, res) => {
    fs.readFile(path.join(compiler.outputPath, 'index.html'), (err, file) => {
      if (err) {
        res.sendStatus(404);
      } else {
        res.send(file.toString());
      }
    });
  });
};
